﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using CompanyData;

namespace CompanySLApp.Web
{
    // NOTE: If you change the class name "CompanyService" here, you must also update the reference to "CompanyService" in Web.config.
    public class CompanyService : ICompanyService
    {

        #region ICompanyService Members

        public List<CompanyData.Companies> GetCompanyList()
        {
            using (CompanyEntities context = new CompanyEntities())
            {
                return (from oneCompany in context.Companies
                        orderby oneCompany.CompanyName
                        select oneCompany).ToList();
            }
            
        }

        public CompanyData.Companies GetCompanyById(int companyID)
        {
            using (CompanyEntities context = new CompanyEntities())
            {
                return (from oneCompany in context.Companies.Include("Persons")
                        where oneCompany.CompanyId == companyID
                        select oneCompany).FirstOrDefault();
            }
        }

        public Companies Update(Companies company)
        {
            using (CompanyEntities context = new CompanyEntities())
            {
                List<Persons> updated = company.Persons.ToList();
                company.Persons.Clear();

                var existing = (from oneCompany in context.Companies.Include("Persons")
                                where oneCompany.CompanyId == company.CompanyId
                                select oneCompany).FirstOrDefault();
                if (existing == null)
                {
                    context.AddToCompanies(company);
                    foreach (var item in updated)
                    {
                        company.Persons.Add(item);
                    }
                }
                else
                {
                    context.ApplyPropertyChanges("Companies", company);
                    List<Persons> deleted = new List<Persons>();
                    foreach (var item in existing.Persons)
                    {
                        var newPerson = (from one in updated
                                         where one.PersonId == item.PersonId
                                         select one).FirstOrDefault();
                        if (newPerson != null)
                        {
                            context.ApplyPropertyChanges("Persons", newPerson);
                        }
                        else
                        {
                            deleted.Add(item);
                        }
                    }
                    foreach (var item in deleted)
                    {
                        context.DeleteObject(item);
                    }
                    foreach (var item in updated)
                    {
                        var oldPerson = (from one in existing.Persons
                                         where one.PersonId == item.PersonId
                                         select one).FirstOrDefault();
                        if (oldPerson == null)
                        {
                            existing.Persons.Add(item);
                        }
                    }
                }


                context.SaveChanges();
                if (existing == null)
                    return company;
                else
                    return existing;
            }
        }

        public void Delete(int companyID)
        {
            using (CompanyEntities context = new CompanyEntities())
            {
                Companies company = (from oneCompany in context.Companies
                                     where oneCompany.CompanyId == companyID
                                     select oneCompany).FirstOrDefault();
                context.DeleteObject(company);
                context.SaveChanges();
            }
        }

        #endregion
    }
}
